{
  "": {
    "domain": "ckan",
    "lang": "de",
    "plural-forms": "nplurals=2; plural=(n != 1);"
  },
  "An Error Occurred": [
    null,
    "Es trat ein Fehler auf"
  ],
  "An error occurred: %(text)s %(error)s": [
    null,
    "Ein Fehler ist aufgetreten: %(text)s %(error)s"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "Bist du sicher, dass du diese Aktion ausführen willst?"
  ],
  "Cancel": [
    null,
    "Abbrechen"
  ],
  "Confirm": [
    null,
    "Bestätigen"
  ],
  "Edit": [
    null,
    "Bearbeiten"
  ],
  "Failed to load data API information": [
    null,
    "Informationen zur Daten-API konnten nicht geladen werden"
  ],
  "Follow": [
    null,
    "Folgen"
  ],
  "Hide": [
    null,
    "Verstecken"
  ],
  "Image": [
    null,
    "Bild"
  ],
  "Input is too short, must be at least one character": [
    null,
    "Eingabe ist zu kurz, muss mindestens ein Zeichen sein"
  ],
  "Link": [
    null,
    "Link"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "Link zu einer URL im Internet (Sie können auch den Link zu einer API angeben)"
  ],
  "Loading...": [
    null,
    "Lädt..."
  ],
  "No matches found": [
    null,
    "Keine Treffer"
  ],
  "Please Confirm Action": [
    null,
    "Bestätige bitte die Aktion"
  ],
  "Remove": [
    null,
    "Entfernen"
  ],
  "Reorder resource view": [
    null,
    "Ressourcen neu ordnen"
  ],
  "Reorder resources": [
    null,
    "Ressourcen neu ordnen"
  ],
  "Resource uploaded": [
    null,
    "Ressource hochgeladen"
  ],
  "Save order": [
    null,
    "Reihenfolge speichern"
  ],
  "Saving...": [
    null,
    "Speichern..."
  ],
  "Show more": [
    null,
    "Mehr anzeigen"
  ],
  "Start typing…": [
    null,
    "Gib etwas ein..."
  ],
  "There are unsaved modifications to this form": [
    null,
    "Dieses Formular hat ungespeicherte Änderungen"
  ],
  "There is no API data to load for this resource": [
    null,
    "Diese Ressource hat keine API-Daten zum Laden"
  ],
  "URL": [
    null,
    "URL"
  ],
  "Unable to authenticate upload": [
    null,
    "Upload konnte nicht autentifiziert werden"
  ],
  "Unable to get data for uploaded file": [
    null,
    "Daten für hochgeladen Datei konnten nicht abgerufen werden"
  ],
  "Unable to upload file": [
    null,
    "Datei konnte nicht hochgeladen werden"
  ],
  "Unfollow": [
    null,
    "Entfolgen"
  ],
  "Upload": [
    null,
    "Hochladen"
  ],
  "Upload a file": [
    null,
    "Datei hochladen"
  ],
  "Upload a file on your computer": [
    null,
    "Datei von Ihrem Computer hochladen"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "Eine Datei wird hochgeladen. Soll das wirklich abgebrochen werden?"
  ],
  "error loading view": [
    null,
    "Fehler beim Laden der Ansicht"
  ],
  "show less": [
    null,
    "weniger anzeigen"
  ],
  "show more": [
    null,
    "mehr anzeigen"
  ]
}