{
  "": {
    "domain": "ckan",
    "lang": "zh_TW",
    "plural-forms": "nplurals=1; plural=0;"
  },
  "An Error Occurred": [
    null,
    "發生錯誤"
  ],
  "An error occurred: %(text)s %(error)s": [
    null,
    "發生錯誤：%(text)s %(error)s"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "確定要展示此活動嗎？"
  ],
  "Cancel": [
    null,
    "取消"
  ],
  "Confirm": [
    null,
    "確認"
  ],
  "Edit": [
    null,
    "編輯"
  ],
  "Failed to load data API information": [
    null,
    "讀取API資訊失敗"
  ],
  "Follow": [
    null,
    "跟隨"
  ],
  "Hide": [
    null,
    "隱藏"
  ],
  "Image": [
    null,
    "圖片"
  ],
  "Input is too short, must be at least one character": [
    null,
    "輸入的內容太短，必須至少輸入一個字元"
  ],
  "Link": [
    null,
    "連結"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "連結至網際網路上的 URL (你也可以連結至 API)"
  ],
  "Loading...": [
    null,
    "讀取中..."
  ],
  "No matches found": [
    null,
    "沒有匹配的資料"
  ],
  "Please Confirm Action": [
    null,
    "請確認活動"
  ],
  "Remove": [
    null,
    "移除"
  ],
  "Reorder resource view": [
    null,
    "重新排序資料檢視"
  ],
  "Reorder resources": [
    null,
    "重新排序資料"
  ],
  "Resource uploaded": [
    null,
    "資源已上傳"
  ],
  "Save order": [
    null,
    "儲存排序"
  ],
  "Saving...": [
    null,
    "儲存中..."
  ],
  "Show more": [
    null,
    "顯示更多"
  ],
  "Start typing…": [
    null,
    "開始輸入.."
  ],
  "There are unsaved modifications to this form": [
    null,
    "此表格有未保存的修改"
  ],
  "There is no API data to load for this resource": [
    null,
    "這個資源中沒有API資料可供讀取"
  ],
  "URL": [
    null,
    "網址"
  ],
  "Unable to authenticate upload": [
    null,
    "無法驗證上傳"
  ],
  "Unable to get data for uploaded file": [
    null,
    "無法取得上傳檔案的資料"
  ],
  "Unable to upload file": [
    null,
    "無法上傳檔案"
  ],
  "Unfollow": [
    null,
    "取消追蹤"
  ],
  "Upload": [
    null,
    "上傳"
  ],
  "Upload a file": [
    null,
    "上傳檔案"
  ],
  "Upload a file on your computer": [
    null,
    "從你的電腦上傳檔案"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "你現在正在上傳一個檔案，確定要離開當前頁面及停止上傳檔案嗎？"
  ],
  "error loading view": [
    null,
    "讀取資料檢視發生錯誤"
  ],
  "show less": [
    null,
    "顯示較少"
  ],
  "show more": [
    null,
    "顯示更多"
  ]
}