{
  "": {
    "domain": "ckan",
    "lang": "bg",
    "plural-forms": "nplurals=2; plural=(n != 1);"
  },
  "An Error Occurred": [
    null,
    "Възникна грешка"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "Сигурни ли сте, че искате да извършите това действие?"
  ],
  "Cancel": [
    null,
    "Отказ"
  ],
  "Confirm": [
    null,
    "Потвърждаване"
  ],
  "Edit": [
    null,
    "Редакция"
  ],
  "Failed to load data API information": [
    null,
    "Неуспешно зареждане на информация за API на данните"
  ],
  "Follow": [
    null,
    "Следване"
  ],
  "Hide": [
    null,
    "Скриване"
  ],
  "Image": [
    null,
    "Изображение"
  ],
  "Input is too short, must be at least one character": [
    null,
    "Въведената информация е твърде кратка, трябва да бъде поне един символ"
  ],
  "Link": [
    null,
    "Връзка"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "Посочване на връзка към URL адрес в интернет (можете също така да посочите връзка към API)"
  ],
  "Loading...": [
    null,
    "Зареждане...."
  ],
  "No matches found": [
    null,
    "Няма намерени съвпадения"
  ],
  "Please Confirm Action": [
    null,
    "Моля, потвърдете действието"
  ],
  "Remove": [
    null,
    "Премахване"
  ],
  "Reorder resources": [
    null,
    "Подреждане на ресурси"
  ],
  "Resource uploaded": [
    null,
    "Ресурсът е качен"
  ],
  "Save order": [
    null,
    "Запазване на подредбата"
  ],
  "Saving...": [
    null,
    "Запазване..."
  ],
  "Show more": [
    null,
    "Показване на още"
  ],
  "Start typing…": [
    null,
    "Започнете да пишете..."
  ],
  "There are unsaved modifications to this form": [
    null,
    "Има незапазени промени в този формуляр"
  ],
  "There is no API data to load for this resource": [
    null,
    "Няма информация в API за този ресурс"
  ],
  "URL": [
    null,
    "URL адрес"
  ],
  "Unable to authenticate upload": [
    null,
    "Неуспешна проверката за автентичност на качения файл"
  ],
  "Unable to get data for uploaded file": [
    null,
    "Неуспешно зареждане на информацията за качения файл"
  ],
  "Unable to upload file": [
    null,
    "Неуспешно качване на файл"
  ],
  "Unfollow": [
    null,
    "Прекратяване на следването"
  ],
  "Upload": [
    null,
    "Качване"
  ],
  "Upload a file": [
    null,
    "Качване на файл"
  ],
  "Upload a file on your computer": [
    null,
    "Качване файл от Вашия компютър"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "В момента качвате файл. Сигурни ли сте, че искате да излезете от страницата и да прекъснете качването?"
  ],
  "show less": [
    null,
    "показване на по-малко"
  ],
  "show more": [
    null,
    "показване на повече"
  ]
}