{
  "": {
    "domain": "ckan",
    "lang": "el",
    "plural-forms": "nplurals=2; plural=(n != 1);"
  },
  "An Error Occurred": [
    null,
    "Παρουσιάστε ένα λάθος"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "Είστε βέβαιοι ότι θέλετε να εκτελέσετε αυτή την ενέργεια;"
  ],
  "Cancel": [
    null,
    "Ακύρωση"
  ],
  "Confirm": [
    null,
    "Επιβεβαίωση"
  ],
  "Edit": [
    null,
    "Επεξεργασία"
  ],
  "Failed to load data API information": [
    null,
    "Αποτυχία φόρτωσης πληροφοριών από το API δεδομένων"
  ],
  "Follow": [
    null,
    "Ακολουθήστε"
  ],
  "Hide": [
    null,
    "Απόκρυψη"
  ],
  "Image": [
    null,
    "Εικόνα"
  ],
  "Input is too short, must be at least one character": [
    null,
    "Δεδομένα εισόδου πολύ μικρά, πρέπει να είναι τουλάχιστον ένας χαρακτήρας"
  ],
  "Link": [
    null,
    "Σύνδεσμος"
  ],
  "Loading...": [
    null,
    "Φορτώνει...."
  ],
  "No matches found": [
    null,
    "Δεν βρέθηκε"
  ],
  "Please Confirm Action": [
    null,
    "Παρακαλώ Επιβεβαιώστε "
  ],
  "Remove": [
    null,
    "Αφαίρεση"
  ],
  "Reorder resources": [
    null,
    "Επαναταξινόμηση πόρων"
  ],
  "Resource uploaded": [
    null,
    "O πόρος μεταφορτώθηκε επιτυχώς"
  ],
  "Save order": [
    null,
    "Αποθήκευση ταξινόμησης"
  ],
  "Saving...": [
    null,
    "Αποθήκευση..."
  ],
  "Show more": [
    null,
    "Προβολή περισσοτέρων"
  ],
  "Start typing…": [
    null,
    "Ξεκινήστε να πληκτρολογείτε..."
  ],
  "There are unsaved modifications to this form": [
    null,
    "Υπάρχουν μη αποθηκευμένες αλλαγές στην φόρμα"
  ],
  "There is no API data to load for this resource": [
    null,
    "Δεν υπάρχουν δεδομένα από το API για αυτόν τον πόρο"
  ],
  "URL": [
    null,
    "URL"
  ],
  "Unable to authenticate upload": [
    null,
    "Αδυναμία επικύρωσης της μεταφόρτωσης"
  ],
  "Unable to get data for uploaded file": [
    null,
    "Αδυναμία λήψης δεδομένων του αρχείου μεταφόρτωσης"
  ],
  "Unable to upload file": [
    null,
    "Αδυναμία μεταφόρτωσης του αρχείου"
  ],
  "Unfollow": [
    null,
    "Διακοπή ακολούθησης"
  ],
  "Upload": [
    null,
    "Ανεβάστε"
  ],
  "Upload a file": [
    null,
    "Φορτώστε ένα αρχείο"
  ],
  "Upload a file on your computer": [
    null,
    "Μεταφόρτωση αρχείου στον υπολογιστή σας"
  ],
  "show less": [
    null,
    "λιγότερα"
  ],
  "show more": [
    null,
    "περισσότερα"
  ]
}