{% extends "package/read_base.html" %}


{% block subtitle %}{{ _('Related') }} - {{ super() }}{% endblock %}

{% block primary_content_inner %}
  <h2 class="hide-heading">{% block page_heading %}{{ _('Related Media for {dataset}').format(dataset=h.dataset_display_name(c.pkg)) }}{% endblock %}</h2>
  {% block related_list %}
    {% if c.related_list %}
      {% snippet "related/snippets/related_list.html", related_items=c.related_list, pkg_id=c.pkg_dict.name %}
    {% else %}
      <p class="empty">{{ _('No related items') }}</p>
    {% endif %}
  {% endblock %}
  {% block form_actions %}
    <div class="form-actions">
      {% link_for _('Add Related Item'), controller='related', action='new', id=pkg.name, class_='btn btn-primary' %}
    </div>
  {% endblock %}
{% endblock %}
