{
  "": {
    "domain": "ckan",
    "lang": "pt_BR",
    "plural-forms": "nplurals=2; plural=(n > 1);"
  },
  "An Error Occurred": [
    null,
    "Ocorreu um erro"
  ],
  "An error occurred: %(text)s %(error)s": [
    null,
    "Ocorreu um erro: %(text)s %(error)s"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "Você tem certeza de que deseja executar essa ação?"
  ],
  "Cancel": [
    null,
    "Cancelar"
  ],
  "Confirm": [
    null,
    "Confirme"
  ],
  "Edit": [
    null,
    "Editar"
  ],
  "Failed to load data API information": [
    null,
    "Falhou em carregar informações sobre a API de dados"
  ],
  "Follow": [
    null,
    "Seguir"
  ],
  "Hide": [
    null,
    "Ocultar"
  ],
  "Image": [
    null,
    "Imagem"
  ],
  "Input is too short, must be at least one character": [
    null,
    "Entrada insuficiente. É necessário ao menos um caractere."
  ],
  "Link": [
    null,
    "Link"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "Link para uma URL na internet (você também pode ligar a uma API)"
  ],
  "Loading...": [
    null,
    "Carregando ..."
  ],
  "No matches found": [
    null,
    "Nenhum resultado encontrado"
  ],
  "Please Confirm Action": [
    null,
    "Favor confirmar a ação"
  ],
  "Remove": [
    null,
    "Remover"
  ],
  "Reorder resource view": [
    null,
    "Reordenar visão de recurso"
  ],
  "Reorder resources": [
    null,
    "Reordenar recursos"
  ],
  "Resource uploaded": [
    null,
    "Recurso enviado"
  ],
  "Save order": [
    null,
    "Salvar ordem"
  ],
  "Saving...": [
    null,
    "Salvando..."
  ],
  "Show more": [
    null,
    "Mostrar mais"
  ],
  "Start typing…": [
    null,
    "Comece a digitar ..."
  ],
  "There are unsaved modifications to this form": [
    null,
    "Há modificações não salvas neste formulário"
  ],
  "There is no API data to load for this resource": [
    null,
    "Não existe API de dados para carregar para este recurso"
  ],
  "URL": [
    null,
    "URL"
  ],
  "Unable to authenticate upload": [
    null,
    "Não foi possível autenticar o envio"
  ],
  "Unable to get data for uploaded file": [
    null,
    "Não foi possível obter os dados do arquivo carregado"
  ],
  "Unable to upload file": [
    null,
    "Não foi possível enviar o arquivo"
  ],
  "Unfollow": [
    null,
    "Deixar de seguir"
  ],
  "Upload": [
    null,
    "Enviar"
  ],
  "Upload a file": [
    null,
    "Carrega um arquivo"
  ],
  "Upload a file on your computer": [
    null,
    "Enviar um arquivo do seu computador"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "Você está enviando um arquivo. Tem certeza de que quer navegar para outra página e parar esse envio?"
  ],
  "error loading view": [
    null,
    "erro ao carregar visualização"
  ],
  "show less": [
    null,
    "mostrar menos"
  ],
  "show more": [
    null,
    "mostrar mais"
  ]
}