{
  "": {
    "domain": "ckan",
    "lang": "pt_PT",
    "plural-forms": "nplurals=2; plural=(n != 1);"
  },
  "An Error Occurred": [
    null,
    "Ocorreu Um Erro"
  ],
  "An error occurred: %(text)s %(error)s": [
    null,
    "Ocorreu um erro: %(text)s %(error)s"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "Tem a certeza que deseja realizar esta ação?"
  ],
  "Cancel": [
    null,
    "Cancelar"
  ],
  "Confirm": [
    null,
    "Confirmar"
  ],
  "Edit": [
    null,
    "Editar"
  ],
  "Failed to load data API information": [
    null,
    "Falha no carregamento da informação dos dados API"
  ],
  "Follow": [
    null,
    "Seguir"
  ],
  "Hide": [
    null,
    "Ocultar"
  ],
  "Image": [
    null,
    "Imagem"
  ],
  "Input is too short, must be at least one character": [
    null,
    "Entrada muito curta, deve ter pelo menos um caratere."
  ],
  "Link": [
    null,
    "Hiperligação"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "Link para um URL na internet ( pode também ligar-se com API)"
  ],
  "Loading...": [
    null,
    "A carregar ..."
  ],
  "No matches found": [
    null,
    "Não foram encontradas correspondências"
  ],
  "Please Confirm Action": [
    null,
    "Por favor, Confirme a Ação"
  ],
  "Remove": [
    null,
    "Remover"
  ],
  "Reorder resource view": [
    null,
    "Reordenar as vistas dos recursos"
  ],
  "Reorder resources": [
    null,
    "Reordenar recursos"
  ],
  "Resource uploaded": [
    null,
    "Recurso enviado"
  ],
  "Save order": [
    null,
    "Guardar ordenação"
  ],
  "Saving...": [
    null,
    "A guardar ..."
  ],
  "Show more": [
    null,
    "Mostrar mais"
  ],
  "Start typing…": [
    null,
    "Comece a escrever ..."
  ],
  "There are unsaved modifications to this form": [
    null,
    "Existem modificações que não estão a ser guardadas"
  ],
  "There is no API data to load for this resource": [
    null,
    "Não há dados API para carregar neste recurso"
  ],
  "URL": [
    null,
    "URL"
  ],
  "Unable to authenticate upload": [
    null,
    "Não é possivel autenticar o envio"
  ],
  "Unable to get data for uploaded file": [
    null,
    "Não foi possivel obter dados a partir do ficheiro enviado"
  ],
  "Unable to upload file": [
    null,
    "Não é possivel enviar o ficheiro"
  ],
  "Unfollow": [
    null,
    "Não Seguir"
  ],
  "Upload": [
    null,
    "Enviar"
  ],
  "Upload a file": [
    null,
    "Enviar um ficheiro"
  ],
  "Upload a file on your computer": [
    null,
    "Envie um ficheiro no seu computador"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "Está a enviar um ficheiro. Tem a certeza de que deseja sair e parar o envio?"
  ],
  "error loading view": [
    null,
    "erro a carregar a visualização"
  ],
  "show less": [
    null,
    "mostrar menos"
  ],
  "show more": [
    null,
    "mostrar mais"
  ]
}