{% extends "page.html" %}

{% block subtitle %}{{ _("Confirm Delete") }}{% endblock %}

{% block maintag %}<div class="row" role="main">{% endblock %}

{% block main_content %}
  <section class="module span6 offset3">
    <div class="module-content">
      {% block form %}
        <p>{{ _('Are you sure you want to delete member - {name}?').format(name=c.user_dict.name) }}</p>
        <p class="form-actions">
          <form action="{% url_for controller='organization', action='member_delete', id=c.group_id %}" method="post">
            <input type="hidden" name="user" value="{{ c.user_id }}" />
            <button class="btn" type="submit" name="cancel" >{{ _('Cancel') }}</button>
            <button class="btn btn-primary" type="submit" name="delete" >{{ _('Confirm Delete') }}</button>
          </form>
        </p>
      {% endblock %}
    </div>
  </section>
{% endblock %}
