__version__ = '2.5.7'

__description__ = 'CKAN Software'
__long_description__ = \
'''CKAN software provides a hub for datasets. The flagship site running CKAN
is theDataHub.org but it is also used for dozens of other open data websites
run by governments, agencies and citizens.

CKAN provides a place to search for open knowledge resources as well as
register your own - be that a set of Shakespeare's works, a global
population density database, the voting records of MPs, or 30 years of
US patents.

Those familiar with Freshmeat or CPAN can think of CKAN as providing an
analogous service for open data and knowledge.
'''
__license__ = 'AGPL'

# The packaging system relies on this import, please do not remove it
import sys; sys.path.insert(0, __path__[0])
