{% extends "page.html" %}

{% set organization = c.group_dict %}

{% block subtitle %}{{ c.group_dict.display_name }} - {{ _('Organizations') }}{% endblock %}

{% block breadcrumb_content %}
  <li>{% link_for _('Organizations'), controller='organization', action='index' %}</li>
  {% block breadcrumb_content_inner %}
    <li>{% link_for organization.display_name|truncate(35), controller='organization', action='read', id=organization.name %}</li>
    <li class="active">{% link_for _('Manage'), controller='organization', action='edit', id=organization.name %}</li>
  {% endblock %}
{% endblock %}

{% block content_action %}
  {% if organization and h.check_access('organization_update', {'id': organization.id}) %}
    {% link_for _('View'), controller='organization', action='read', id=organization.name, class_='btn', icon='eye-open' %}
  {% endif %}
{% endblock %}

{% block content_primary_nav %}
  {{ h.build_nav_icon('organization_edit', _('Edit'), id=c.group_dict.name) }}
  {{ h.build_nav_icon('organization_bulk_process', _('Datasets'), id=c.group_dict.name) }}
  {{ h.build_nav_icon('organization_members', _('Members'), id=c.group_dict.name) }}
{% endblock %}

{% block secondary_content %}
  {% if organization %}
    {% snippet 'snippets/organization.html', organization=organization %}
  {% else %}
    {{ super() }}
  {% endif %}
{% endblock %}

{% block links %}
  {{ super() }}
  {% include "organization/snippets/feeds.html" %}
{% endblock %}
