{% extends "package/base.html" %}

{% block subtitle %}{{ pkg.title or pkg.name }} - {{ super() }}{% endblock %}

{% block head_extras -%}
  {{ super() }}
  {% set description = h.markdown_extract(pkg.notes, extract_length=200)|forceescape %}
  <meta property="og:title" content="{{ h.dataset_display_name(pkg) }} - {{ g.site_title }}">
  <meta property="og:description" content="{{ description|forceescape|trim }}">
{% endblock -%}

{% block content_action %}
  {% if h.check_access('package_update', {'id':pkg.id }) %}
    {% link_for _('Manage'), controller='package', action='edit', id=pkg.name, class_='btn', icon='wrench' %}
  {% endif %}
{% endblock %}

{% block content_primary_nav %}
  {{ h.build_nav_icon('dataset_read', _('Dataset'), id=pkg.name) }}
  {{ h.build_nav_icon('dataset_groups', _('Groups'), id=pkg.name) }}
  {{ h.build_nav_icon('dataset_activity', _('Activity Stream'), id=pkg.name) }}
{% endblock %}

{% block primary_content_inner %}
    {% block package_revision_info %}
      {% if c.revision_date %}
        <div class="module info alert alert-info">
          <p class="module-content">
            {% set timestamp = h.render_datetime(c.revision_date, with_hours=True) %}
            {% set url = h.url(controller='package', action='read', id=pkg.name) %}

            {% trans timestamp=timestamp, url=url %}This is an old revision of this dataset, as edited at {{ timestamp }}. It may differ significantly from the <a href="{{ url }}">current revision</a>.{% endtrans %}
          </p>
        </div>
      {% endif %}
    {% endblock %}
{% endblock %}

{% block secondary_content %}

  {% block secondary_help_content %}{% endblock %}

  {% block package_info %}
    {% snippet 'package/snippets/info.html', pkg=pkg %}
  {% endblock %}

  {% block package_organization %}
    {% if pkg.organization %}
      {% set org = h.get_organization(pkg.organization.name) %}
      {% snippet "snippets/organization.html", organization=org, has_context_title=true %}
    {% endif %}
  {% endblock %}

  {% block package_social %}
    {% snippet "snippets/social.html" %}
  {% endblock %}

  {% block package_license %}
    {% snippet "snippets/license.html", pkg_dict=pkg %}
  {% endblock %}

{% endblock %}
