{
  "": {
    "domain": "ckan",
    "lang": "fr",
    "plural-forms": "nplurals=2; plural=(n > 1);"
  },
  "An Error Occurred": [
    null,
    "Erreur"
  ],
  "An error occurred: %(text)s %(error)s": [
    null,
    "Une erreur est survenue : %(text)s %(error)s"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "Etes-vous sûr de vouloir exécuter cette commande ?"
  ],
  "Cancel": [
    null,
    "Annuler"
  ],
  "Confirm": [
    null,
    "Confirmation"
  ],
  "Edit": [
    null,
    "Éditer"
  ],
  "Failed to load data API information": [
    null,
    "Le chargement de l'information de l'API de données a échoué"
  ],
  "Follow": [
    null,
    "S'abonner"
  ],
  "Hide": [
    null,
    "Cacher"
  ],
  "Image": [
    null,
    "Image"
  ],
  "Input is too short, must be at least one character": [
    null,
    "L'entrée est trop courte ; elle doit avoir au moins un caractère"
  ],
  "Link": [
    null,
    "Lien"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "Lien vers une URL sur internet (vous pouvez aussi donner un lien vers une API)"
  ],
  "Loading...": [
    null,
    "Chargement..."
  ],
  "No matches found": [
    null,
    "Aucune correspondance trouvée"
  ],
  "Please Confirm Action": [
    null,
    "Veuillez confirmer l'action"
  ],
  "Remove": [
    null,
    "Supprimer"
  ],
  "Reorder resource view": [
    null,
    "Réordonner la visualisation de ressource"
  ],
  "Reorder resources": [
    null,
    "Réordonner les ressources"
  ],
  "Resource uploaded": [
    null,
    "Ressource  téléchargée"
  ],
  "Save order": [
    null,
    "Sauvegarder la commande"
  ],
  "Saving...": [
    null,
    "Sauvegarde en cours..."
  ],
  "Show more": [
    null,
    "Montrer plus"
  ],
  "Start typing…": [
    null,
    "Commencez la saisie ..."
  ],
  "There are unsaved modifications to this form": [
    null,
    "Des modifications de ce formulaire n'ont pas été enregistrées"
  ],
  "There is no API data to load for this resource": [
    null,
    "Il n'y a pas de données API à télécharger pour cette ressource"
  ],
  "URL": [
    null,
    "URL"
  ],
  "Unable to authenticate upload": [
    null,
    "Échec de l'authentification du transfert"
  ],
  "Unable to get data for uploaded file": [
    null,
    "Impossible d'accéder aux données du fichier déposé"
  ],
  "Unable to upload file": [
    null,
    "Échec du transfert de fichier "
  ],
  "Unfollow": [
    null,
    "Se désabonner"
  ],
  "Upload": [
    null,
    "Envoi"
  ],
  "Upload a file": [
    null,
    "Envoyer un fichier depuis votre ordinateur"
  ],
  "Upload a file on your computer": [
    null,
    "Télécharger un fichier sur votre ordinateur"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "Vous êtes en train de transférer un fichier. Êtes-vous sûr de vouloir naviguer ailleurs et interrompre ce transfert ?"
  ],
  "error loading view": [
    null,
    "erreur lors du chargement de la vue"
  ],
  "show less": [
    null,
    "montrer moins"
  ],
  "show more": [
    null,
    "montrer plus"
  ]
}