{% extends "admin/base.html" %}

{% block primary_content_inner %}
  {% set truncate = truncate or 180 %}
  {% set truncate_title = truncate_title or 80 %}
  <ul class="user-list">
    {% for pkg in c.deleted_packages %}
      {% set title = pkg.title or pkg.name %}
      <li>{{ h.link_to(h.truncate(title, truncate_title), h.url_for(controller='package', action='read', id=pkg.name)) }}</li>
    {% endfor %}

  </ul>
  <form method="POST" id="form-purge-packages">
    <button
      type="submit"
      name="purge-packages"
      value="purge"
      class="btn btn-danger"
      >
      {% trans %}Purge{% endtrans %}
    </button>
  </form>
{% endblock %}

{% block secondary_content %}
  <div class="module module-narrow module-shallow">
    <h2 class="module-heading">
      <i class="icon-info-sign"></i>
      {{ _('Trash') }}
    </h2>
    <div class="module-content">
      {% trans %}
        <p>Purge deleted datasets forever and irreversibly.</p>
      {% endtrans %}
    </div>
  </div>
{% endblock %}

