{
  "": {
    "domain": "ckan",
    "lang": "uk_UA",
    "plural-forms": "nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2);"
  },
  "An Error Occurred": [
    null,
    "Виникла помилка"
  ],
  "An error occurred: %(text)s %(error)s": [
    null,
    "Виникла помилка: %(text)s %(error)s"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "Ви впевнені, що хочете виконати цю операцію?"
  ],
  "Cancel": [
    null,
    "Скасувати"
  ],
  "Confirm": [
    null,
    "Підтвердити"
  ],
  "Edit": [
    null,
    "Редагувати"
  ],
  "Failed to load data API information": [
    null,
    "Не вдалося завантажити інформацію про дані API"
  ],
  "Follow": [
    null,
    "Стежити"
  ],
  "Hide": [
    null,
    "Сховати"
  ],
  "Image": [
    null,
    "Зображення"
  ],
  "Input is too short, must be at least one character": [
    null,
    "Ввід надто короткий, має бути хоча би один символ"
  ],
  "Link": [
    null,
    "Посилання"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "Додати посилання на адресу в інтернеті (або посилання на API)"
  ],
  "Loading...": [
    null,
    "Завантаження..."
  ],
  "No matches found": [
    null,
    "Збігів не знайдено"
  ],
  "Please Confirm Action": [
    null,
    "Підтвердіть операцію"
  ],
  "Remove": [
    null,
    "Видалити"
  ],
  "Reorder resource view": [
    null,
    "Змінити порядок представлення ресурсу"
  ],
  "Reorder resources": [
    null,
    "Змінити порядок ресурсів"
  ],
  "Resource uploaded": [
    null,
    "Ресурс вивантажено"
  ],
  "Save order": [
    null,
    "Зберегти порядок"
  ],
  "Saving...": [
    null,
    "Збереження..."
  ],
  "Show more": [
    null,
    "Показати більше"
  ],
  "Start typing…": [
    null,
    "Друкуйте..."
  ],
  "There are unsaved modifications to this form": [
    null,
    "Ця форма містить незбережені зміни"
  ],
  "There is no API data to load for this resource": [
    null,
    "Для цього ресурсу немає доступних для завантаження даних API"
  ],
  "URL": [
    null,
    "URL"
  ],
  "Unable to authenticate upload": [
    null,
    "Не вдалося завантажити"
  ],
  "Unable to get data for uploaded file": [
    null,
    "Не вдалось отримати дані з вивантаженого файла"
  ],
  "Unable to upload file": [
    null,
    "Не вдалось вивантажити файл"
  ],
  "Unfollow": [
    null,
    "Не стежити"
  ],
  "Upload": [
    null,
    "Вивантажити"
  ],
  "Upload a file": [
    null,
    "Вкласти файл"
  ],
  "Upload a file on your computer": [
    null,
    "Вкласти файл з комп’ютера"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "Ви вкладаєте файл. Ви впевнені, що хочете перейти на іншу сторінку і припинити вкладення?"
  ],
  "error loading view": [
    null,
    "помилка при завантаженні представлення"
  ],
  "show less": [
    null,
    "показати менше"
  ],
  "show more": [
    null,
    "показати більше"
  ]
}