{#
Displays a sidebard module with information for given resource

res    - The respource dict

Example:

  {% snippet "package/snippets/resrouce_info.html", res=res %}

#}
<div class="module context-info">
  <div class="module-content">
    <h1 class="heading">{{ res.name or res.id }}</h1>
    <div class="nums">
      <dl>
        <dt>{{ _('Format') }}</dt>
        <dd>{{ res.format }}</dd>
      </dl>
    </div>
  </div>
</div>
