.datapusher-status-link:hover {
	text-decoration: none;
}

.datapusher-status {
	&.status-unknown {
		color: #bbb;
	}
	&.status-pending {
		color: #FFCC00;
	}
	&.status-error {
		color: red;
	}
	&.status-complete {
		color: #009900;
	}
}