
// Bootstrap imports

// CSS Reset
@import "../vendor/bootstrap/less/reset.less";

// Core variables and mixins
@import "../vendor/bootstrap/less/variables.less"; // Modify this for custom colors, font-sizes, etc
@import "../vendor/bootstrap/less/mixins.less";

// Grid system and page structure
@import "../vendor/bootstrap/less/scaffolding.less";
@import "../vendor/bootstrap/less/grid.less";
@import "../vendor/bootstrap/less/layouts.less";

// Base CSS
@import "../vendor/bootstrap/less/type.less";
@import "../vendor/bootstrap/less/code.less";
@import "../vendor/bootstrap/less/forms.less";
@import "../vendor/bootstrap/less/tables.less";

// Components: common
// @import "../vendor/bootstrap/less/sprites.less"; // Replaced by font-awesome.less
@import "../vendor/bootstrap/less/dropdowns.less";
@import "../vendor/bootstrap/less/wells.less";
@import "../vendor/bootstrap/less/component-animations.less";
@import "../vendor/bootstrap/less/close.less";

// Components: Buttons & Alerts
@import "../vendor/bootstrap/less/buttons.less";
@import "../vendor/bootstrap/less/button-groups.less";
@import "../vendor/bootstrap/less/alerts.less"; // Note: alerts share common CSS with buttons and thus have styles in buttons.less

// Components: Nav
@import "../vendor/bootstrap/less/navs.less";
@import "../vendor/bootstrap/less/navbar.less";
@import "../vendor/bootstrap/less/breadcrumbs.less";
@import "../vendor/bootstrap/less/pagination.less";
@import "../vendor/bootstrap/less/pager.less";

// Components: Popovers
@import "../vendor/bootstrap/less/modals.less";
@import "../vendor/bootstrap/less/tooltip.less";

// Components: Misc
@import "../vendor/bootstrap/less/thumbnails.less";
@import "../vendor/bootstrap/less/labels-badges.less";
@import "../vendor/bootstrap/less/progress-bars.less";
@import "../vendor/bootstrap/less/accordion.less";
@import "../vendor/bootstrap/less/carousel.less";
@import "../vendor/bootstrap/less/hero-unit.less";
@import "../vendor/bootstrap/less/popovers.less";

// Utility classes
@import "../vendor/bootstrap/less/utilities.less"; // Has to be last to override when necessary

@import "../vendor/bootstrap/less/responsive.less";

// CKAN
@import "ckan.less";
