{% extends "group/base_form_page.html" %}

{% block breadcrumb_content %}
  <li>{% link_for _('Groups'), controller='group', action='index' %}</li>
  {% block breadcrumb_content_inner %}
    <li>{% link_for group.display_name|truncate(35), controller='group', action='read', id=group.name %}</li>
    <li class="active">{% link_for _('Manage'), controller='group', action='edit', id=group.name %}</li>
  {% endblock %}
{% endblock %}

{% block page_heading_class %}hide-heading{% endblock %}
{% block page_heading %}{{ _('Edit Group') }}{% endblock %}
