.resource-view-filters {
  margin-bottom: 1em;

  .resource-view-filter {
    margin-bottom: 1.0em;
  }

  .resource-view-remove-filter {
    cursor: pointer;
    color: @errorText;
  }

  .resource-view-filter-values .select2-container {
    margin-right: 0.3em;
    margin-bottom: 0.2em;
    width: 24% !important;

    .select2-search-choice-close {
      left: auto;
    }
  }
}
