{
  "": {
    "domain": "ckan",
    "lang": "nl",
    "plural-forms": "nplurals=2; plural=(n != 1);"
  },
  "An Error Occurred": [
    null,
    "Er is een fout opgetreden"
  ],
  "An error occurred: %(text)s %(error)s": [
    null,
    "Er is een fout opgetreden: %(text)s %(error)s"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "Weet u zeker dat u deze actie uit wil voeren?"
  ],
  "Cancel": [
    null,
    "Afbreken"
  ],
  "Confirm": [
    null,
    "Bevestig"
  ],
  "Edit": [
    null,
    "Bijwerk"
  ],
  "Failed to load data API information": [
    null,
    "Het laden van de API informatie data is mislukt"
  ],
  "Follow": [
    null,
    "Volg"
  ],
  "Hide": [
    null,
    "verberg"
  ],
  "Image": [
    null,
    "Afbeelding"
  ],
  "Input is too short, must be at least one character": [
    null,
    "Uw input is te kort, hij moet uit minimaal een karakter bestaan"
  ],
  "Link": [
    null,
    "Link"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "Gebruik een weblocatie (URL). Dit mag ook de URL van een API zijn."
  ],
  "Loading...": [
    null,
    "Laden..."
  ],
  "No matches found": [
    null,
    "Geen overeenkomsten gevonden"
  ],
  "Please Confirm Action": [
    null,
    "Bevestig de actie"
  ],
  "Remove": [
    null,
    "Verwijder"
  ],
  "Reorder resource view": [
    null,
    "Wijzig sortering bronnen"
  ],
  "Reorder resources": [
    null,
    "Wijzig sortering bronnen"
  ],
  "Resource uploaded": [
    null,
    "De bron is geupload"
  ],
  "Save order": [
    null,
    "Bewaar sortering"
  ],
  "Saving...": [
    null,
    "Opslaan..."
  ],
  "Show more": [
    null,
    "Toon meer"
  ],
  "Start typing…": [
    null,
    "Begin met typen"
  ],
  "There are unsaved modifications to this form": [
    null,
    "Er zijn niet-opgeslagen wijzigingen in dit formulier"
  ],
  "There is no API data to load for this resource": [
    null,
    "Er is geen API data om deze bron te openen"
  ],
  "URL": [
    null,
    "URL"
  ],
  "Unable to authenticate upload": [
    null,
    "Kan de upload niet authenticeren"
  ],
  "Unable to get data for uploaded file": [
    null,
    "Kan geen data vinden voor het geuploaden bestand"
  ],
  "Unable to upload file": [
    null,
    "Kan het bestand niet uploaden"
  ],
  "Unfollow": [
    null,
    "Ontvolgen"
  ],
  "Upload": [
    null,
    "Upload"
  ],
  "Upload a file": [
    null,
    "Upload een bestand"
  ],
  "Upload a file on your computer": [
    null,
    "Upload een bestand"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "Bestand wordt geupload. Als u deze pagina verlaat stopt de upload. Weet u zeker dat u de pagina wilt verlaten?"
  ],
  "show less": [
    null,
    "laat minder zien"
  ],
  "show more": [
    null,
    "laat meer zien"
  ]
}