{
  "": {
    "domain": "ckan",
    "lang": "no",
    "plural-forms": "nplurals=2; plural=(n != 1);"
  },
  "An Error Occurred": [
    null,
    "En feil oppsto"
  ],
  "An error occurred: %(text)s %(error)s": [
    null,
    "En feil har oppstått: %(text)s %(error)s"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "Er du sikker på at du vil utføre denne handlingen?"
  ],
  "Cancel": [
    null,
    "Avbryt"
  ],
  "Confirm": [
    null,
    "Bekreft"
  ],
  "Edit": [
    null,
    "Rediger"
  ],
  "Failed to load data API information": [
    null,
    "Kunne ikke laste informasjon om data-API"
  ],
  "Follow": [
    null,
    "Følg"
  ],
  "Hide": [
    null,
    "Skjul"
  ],
  "Image": [
    null,
    "Bilde"
  ],
  "Input is too short, must be at least one character": [
    null,
    "For kort, må være minst ett tegn"
  ],
  "Link": [
    null,
    "Lenke"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "Lenke til en URL på internett (du kan også lenke til et API)"
  ],
  "Loading...": [
    null,
    "Laster..."
  ],
  "No matches found": [
    null,
    "Ingen treff"
  ],
  "Please Confirm Action": [
    null,
    "Bekreft handling"
  ],
  "Remove": [
    null,
    "Fjern"
  ],
  "Reorder resource view": [
    null,
    "Forandre rekkefølgen i ressursvisning"
  ],
  "Reorder resources": [
    null,
    "Forandre rekkefølge på ressurser"
  ],
  "Resource uploaded": [
    null,
    "Ressurs lastet opp"
  ],
  "Save order": [
    null,
    "Lagre rekkefølge"
  ],
  "Saving...": [
    null,
    "Lagrer..."
  ],
  "Show more": [
    null,
    "Vis mer"
  ],
  "Start typing…": [
    null,
    "Skriv inn..."
  ],
  "There are unsaved modifications to this form": [
    null,
    "Du har gjort endringer i skjemaet som ikke er lagret"
  ],
  "There is no API data to load for this resource": [
    null,
    "Ingen API-data kan lastes for denne ressursen"
  ],
  "URL": [
    null,
    "URL"
  ],
  "Unable to authenticate upload": [
    null,
    "Klarer ikke å autentisere opplasting"
  ],
  "Unable to get data for uploaded file": [
    null,
    "Finner ikke data for opplastet fil"
  ],
  "Unable to upload file": [
    null,
    "Klarer ikke å laste opp fil"
  ],
  "Unfollow": [
    null,
    "Ikke følg"
  ],
  "Upload": [
    null,
    "Opplasting"
  ],
  "Upload a file": [
    null,
    "Last opp en fil"
  ],
  "Upload a file on your computer": [
    null,
    "Last opp en fil til datamaskinen din"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "Du laster opp en fil. Er du sikker på at du vil forlate siden og stoppe denne opplastingen?"
  ],
  "error loading view": [
    null,
    "feil i innlasting av visning"
  ],
  "show less": [
    null,
    "vis mindre"
  ],
  "show more": [
    null,
    "vis mer"
  ]
}