<div class="module module-narrow module-shallow">
  <h2 class="module-heading">
    <i class="icon-large icon-info-sign"></i>
    {{ _('What are Organizations?') }}
  </h2>
  <div class="module-content">
    {% trans %}
      <p>Organizations act like publishing departments for datasets (for
        example, the Department of Health). This means that datasets can be
        published by and belong to a department instead of an individual
        user.</p>
      <p>Within organizations, admins can assign roles and authorise its
        members, giving individual users the right to publish datasets from
        that particular organisation (e.g. Office of National Statistics).</p>
    {% endtrans %}
  </div>
</div>
