{% extends "page.html" %}

{% block secondary_content %}

  {% block secondary_help_content %}{% endblock %}

  {% block package_social %}
    {% snippet "snippets/social.html" %}
  {% endblock %}

{% endblock %}

{% block primary_content %}
  <article class="module prose">
    <section class="module-content">
      {% block primary_content_inner %}{% endblock %}
    </section>
  </article>
{% endblock %}